@@ -23,8 +23,6 @@ import re
 import unicodedata
 import urllib
 
-import pyparsing as pp
-
 from oslo_utils._i18n import _
 from oslo_utils import encodeutils
 
@@ -39,11 +37,24 @@ UNIT_PREFIX_EXPONENT = {
     'Gi': 3,
     'T': 4,
     'Ti': 4,
+    'P': 5,
+    'Pi': 5,
+    'E': 6,
+    'Ei': 6,
+    'Z': 7,
+    'Zi': 7,
+    'Y': 8,
+    'Yi': 8,
+    'R': 9,
+    'Ri': 9,
+    'Q': 10,
+    'Qi': 10,
 }
 UNIT_SYSTEM_INFO = {
-    'IEC': (1024, re.compile(r'(^[-+]?\d*\.?\d+)([KMGT]i?)?(b|bit|B)$')),
-    'SI': (1000, re.compile(r'(^[-+]?\d*\.?\d+)([kMGT])?(b|bit|B)$')),
-    'mixed': (None, re.compile(r'(^[-+]?\d*\.?\d+)([kKMGT]i?)?(b|bit|B)$')),
+    'IEC': (1024, re.compile(r'(^[-+]?\d*\.?\d+)([KMGTPEZYRQ]i?)?(b|bit|B)$')),
+    'SI': (1000, re.compile(r'(^[-+]?\d*\.?\d+)([kMGTPEZYRQ])?(b|bit|B)$')),
+    'mixed': (None, re.compile(
+        r'(^[-+]?\d*\.?\d+)([kKMGTPEZYRQ]i?)?(b|bit|B)$')),
 }
 
 TRUE_STRINGS = ('1', 't', 'true', 'on', 'y', 'yes')
@@ -79,6 +90,8 @@ _SANITIZE_PATTERNS_WILDCARD = {}
 # have two parameters. Use different lists of patterns here.
 _FORMAT_PATTERNS_1 = [r'(%(key)s[0-9]*\s*[=]\s*)[^\s^\'^\"]+']
 _FORMAT_PATTERNS_2 = [r'(%(key)s[0-9]*\s*[=]\s*[\"\'])[^\"\']*([\"\'])',
+                      r'(%(key)s[0-9]*\s*[=]\s*[\"])[^\"]*([\"])',
+                      r'(%(key)s[0-9]*\s*[=]\s*[\'])[^\']*([\'])',
                       r'(%(key)s[0-9]*\s+[\"\'])[^\"\']*([\"\'])',
                       r'([-]{2}%(key)s[0-9]*\s+)[^\'^\"^=^\s]+([\s]*)',
                       r'(<%(key)s[0-9]*>)[^<]*(</%(key)s[0-9]*>)',
@@ -182,13 +195,19 @@ def string_to_bytes(text, unit_system='IEC', return_int=False):
 
     The units supported for IEC / mixed::
 
-        Kb(it), Kib(it), Mb(it), Mib(it), Gb(it), Gib(it), Tb(it), Tib(it)
-        KB, KiB, MB, MiB, GB, GiB, TB, TiB
+        Kb(it), Kib(it), Mb(it), Mib(it), Gb(it), Gib(it), Tb(it), Tib(it),
+        Pb(it), Pib(it), Eb(it), Eib(it), Zb(it), Zib(it), Yb(it), Yib(it),
+        Rb(it), Rib(it), Qb(it), Qib(it)
+
+        KB, KiB, MB, MiB, GB, GiB, TB, TiB, PB, PiB, EB, EiB, ZB, ZiB,
+        YB, YiB, RB, RiB, QB, QiB
 
     The units supported for SI ::
 
-        kb(it), Mb(it), Gb(it), Tb(it)
-        kB, MB, GB, TB
+        kb(it), Mb(it), Gb(it), Tb(it), Pb(it), Eb(it), Zb(it), Yb(it),
+        Rb(it), Qb(it)
+
+        kB, MB, GB, TB, PB, EB, ZB, YB, RB, QB
 
     SI units are interpreted as power-of-ten (e.g. 1kb = 1000b).  Note
     that the SI unit system does not support capital letter 'K'
@@ -301,8 +320,6 @@ def mask_password(message, secret="***"):  # nosec
     '"password" : "***"'
     >>> mask_password("'original_password' : 'aaaaa'")
     "'original_password' : '***'"
-    >>> mask_password("u'original_password' :   u'aaaaa'")
-    "u'original_password' :   u'***'"
 
     .. versionadded:: 0.2
 
@@ -575,8 +592,13 @@ def split_by_commas(value):
 
     .. versionadded:: 3.17
     """
-    word = (pp.QuotedString(quoteChar='"', escChar='\\') |
-            pp.Word(pp.printables, excludeChars='",'))
+    # pyparsing is a slow import; defer loading until we need it
+    import pyparsing as pp
+
+    word = (
+        pp.QuotedString(quoteChar='"', escChar='\\') |
+        pp.Word(pp.printables, excludeChars='",')
+    )
     grammar = pp.stringStart + pp.delimitedList(word) + pp.stringEnd
 
     try:
